// Reference: http://bl.ocks.org/zwang155/5079c689233607dd9bdd28d4e39fb290

function main() {
  let xLabels = [
    'Suzuki',
    'Subaru',
    'Lotus',
    'Mazda',
    'Hyundai',
    'Honda',
    'Kia',
    'Scion',
    'MINI',
  ];
  let svg = d3.select('svg'),
    margin = { top: 60, bottom: 60, left: 60, right: 0 },
    width = 600 - margin.left - margin.right,
    height = 475 - margin.top - margin.bottom;

  
  //Add Scales
  let xScale = d3
      .scaleBand()
      .range([margin.left, margin.left + width])
      .padding(0.25),
    yScale = d3
      .scaleLinear()
      .range([height + margin.top, margin.top]),
    colorScale = d3
      .scaleQuantize()
		  .domain([10, 100])
    	.range([1, 2, 4])
      .range(colorbrewer.Reds[5]);



  //Add Tooltip
  var div = d3
    .select('body')
    .append('div')
    .attr('class', 'tooltip')
    .style('opacity', 0)
  	.style("font-size", "13px");

  //Decimal number format
  var formatDecimal = d3.format(',.2f');

  //Read Data
  d3.csv('barchart.csv', (d) => {
    return {
      code: d['Make'],
      horsepower: +d[
        'MPG'
      ],
    };
  }).then((dataset) => {
    let data = [];
    dataset.forEach((elem) => {
      if (xLabels.includes(elem.code)) {
        data.push(elem);
      }
    });
    data.sort((a, b) => a.horsepower - b.horsepower);
    let xSeq = [];
    data.forEach((elem) => xSeq.push(elem.code));

    xScale.domain(xSeq);
    yScale.domain([0, d3.max(data, (d) => d.horsepower)]);
    colorScale.domain([
      0,
      d3.max(data, (d) => d.horsepower),
    ]);

    //Add Graph
    svg
      .selectAll('rect')
      .data(data)
      .enter()
      .append('rect')
      .attr('x', (d) => xScale(d.code))
      .attr('y', (d) => yScale(d.horsepower))
      .attr('width', xScale.bandwidth())
      .attr(
        'height',
        (d) => height + margin.top - yScale(d.horsepower)
      )
      .attr('fill', (d) => colorScale(d.horsepower))
      .on('mouseover', function (event, d) {
        div
          .transition()
          .duration(200)
          .style('opacity', 0.9);
        div
          .html(
            'City MPG: <br/>' +
              d.code +
              ': ' +
              formatDecimal(d.horsepower)
          )
          .style('left', event.pageX + 'px')
          .style('top', event.pageY - 28 + 'px');
      })
      .on('mouseout', function (d) {
        div.transition().duration(500).style('opacity', 0);
      });

    //Add Title
    svg
      .append('text')
      .attr('text-anchor', 'middle')
      .attr('x', width / 2 + margin.left)
      .attr('y', margin.top / 2)
      .text('Highest MPG Vehicles by Make');

    //X Axis
    svg
      .append('g')
      .attr('class', 'axis')
      .attr(
        'transform',
        'translate(0, ' + (height + margin.top) + ')'
      )
      .call(d3.axisBottom(xScale).tickValues(xLabels))
      .append('text')
      .attr('text-anchor', 'middle')
      .attr('x', width / 2 + margin.left)
      .attr('y', margin.bottom / 1.5)     
      .text('Make')
    	.style("font-size", "16px");

    //Y Axis
    svg
      .append('g')
      .attr('class', 'axis')
      .attr(
        'transform',
        'translate(' + margin.left + ', 0)'
      )
      .call(
        d3
          .axisLeft(yScale)
          .ticks(5)
          .tickFormat(d3.format('~s'))
      )
      .append('text')
      .attr('text-anchor', 'middle')
      .attr('transform', 'rotate(-90)')
      .attr('x', -height / 2 - margin.top)
      .attr('y', -margin.left / 1.25) 
    .style("font-size", "16px") 
      .text(
        'City MPG'
      );

    //Add Legend
    let legend = d3
      .legendColor()
      .labelFormat(d3.format('.1s'))
      .title('Bar chart Legend')
      .scale(colorScale);
    svg
      .append('g')
      .attr('class', 'legend')
      .attr(
        'transform',
        'translate(' +
          (width + margin.left + margin.right / 12) +
          ',' +
          (height - margin.bottom) / 2 +
          ')'
      )
      .call(legend);
  });
}

main();
